using System;

namespace Bespoke.Robotics.BrainStem
{
	/// <summary>
	/// 
	/// </summary>
	public class ServoConfiguration
	{
		/// <summary>
		/// 
		/// </summary>
		public const byte MIN_SERVO_SPEED = 0;

		/// <summary>
		/// 
		/// </summary>
		public const byte MAX_SERVO_SPEED = 15;

		/// <summary>
		/// 
		/// </summary>
		public ServoIdType ServoId
		{
			get
			{
				return mServoId;
			}
			set
			{
				mServoId = value;
			}
		}

		/// <summary>
		/// 
		/// </summary>
		public bool Enabled
		{
			get
			{
				return mEnabled;
			}
			set
			{
				mEnabled = value;
			}
		}

		/// <summary>
		/// 
		/// </summary>
		public bool Inverted
		{
			get
			{
				return mInverted;
			}
			set
			{
				mInverted = value;
			}
		}

		/// <summary>
		/// 
		/// </summary>
		public byte Speed
		{
			get
			{
				return mSpeed;
			}
			set
			{
				if ((mSpeed < MIN_SERVO_SPEED) || (mSpeed > MAX_SERVO_SPEED))
				{
					throw new ArgumentOutOfRangeException("value", value, string.Format("Valid range between {0}-{1}", MIN_SERVO_SPEED, MAX_SERVO_SPEED));
				}

				mSpeed = value;
			}
		}

		/// <summary>
		/// 
		/// </summary>
		/// <param name="servoId"></param>
		public ServoConfiguration(ServoIdType servoId)
		{
			mServoId = servoId;
			mEnabled = false;
			mInverted = false;
			mSpeed = 0;
		}

		/// <summary>
		/// 
		/// </summary>
		/// <param name="servoId"></param>
		/// <param name="enabled">Enables or disables the servo.</param>
		/// <param name="inverted">Inverts all servo positioning calculations.</param>
		/// <param name="speed">The speed may range from 0-15. A zero turns off the speed feature
		/// and the servo moves immediately from its previous position to any new position. A speed from 1 to 15,
		/// where 1 is the slowest, makes the servo ramp-up from the previous position and ramp-down to any new position.</param>
		/// <exception cref="ArgumentOutOfRangeException">Thrown if <paramref name="speed"/> is out of range.</exception>
		public ServoConfiguration(ServoIdType servoId, bool enabled, bool inverted, byte speed)
		{
			if ((speed < MIN_SERVO_SPEED) || (speed > MAX_SERVO_SPEED))
			{
				throw new ArgumentOutOfRangeException("speed", speed, string.Format("Valid range between {0}-{1}", MIN_SERVO_SPEED, MAX_SERVO_SPEED));
			}

			mServoId = servoId;
			mEnabled = enabled;
			mInverted = inverted;
			mSpeed = speed;
		}

		/// <summary>
		/// 
		/// </summary>
		/// <param name="servoId"></param>
		/// <param name="configurationFlags"></param>
		public ServoConfiguration(ServoIdType servoId, ServoConfigurationFlags configurationFlags)
		{
			mServoId = servoId;
			mEnabled = ServoConfiguration.IsFlagOn(configurationFlags, ServoConfigurationFlags.Enabled);
			mInverted = ServoConfiguration.IsFlagOn(configurationFlags, ServoConfigurationFlags.Inverted);
			mSpeed = (byte)((byte)configurationFlags & 0x0F);
		}

		/// <summary>
		/// 
		/// </summary>
		/// <returns></returns>
		public byte ToDataByte()
		{
			byte dataByte = mSpeed;
			if (mEnabled)
			{
				dataByte |= (byte)ServoConfigurationFlags.Enabled;
			}

			if (mInverted)
			{
				dataByte |= (byte)ServoConfigurationFlags.Inverted;
			}

			return dataByte;
		}

		/// <summary>
		/// 
		/// </summary>
		/// <param name="flags"></param>
		/// <param name="flagToTest"></param>
		/// <returns></returns>
		public static bool IsFlagOn(ServoConfigurationFlags flags, ServoConfigurationFlags flagToTest)
		{
			return ((flags & flagToTest) != 0);
		}

		private ServoIdType mServoId;
		private bool mEnabled;
		private bool mInverted;
		private byte mSpeed;
	}
}
