using System;
using System.Threading;

using Bespoke.Robotics.Common;
using Bespoke.Robotics.BrainStem;
using Bespoke.Robotics.BrainStem.Configuration;

namespace Bespoke.Robotics.BrainStem.Example
{
    public class Program
    {
        static void Main(string[] args)
        {
            try
            {
                // Read the BrainStem module address from the Bespoke.Robotics.BrainStem.config file.
                mModuleAddress = Configuration.Configuration.BrainStem.ModuleAddress;

                // Create and open a new Stream object. For initializing the BrainStem.NET CommandManager.
                mStream = new Stream();
                mStream.Open();

                // This is the last call required before sending commands to the BrainStem.
                CommandManager.Init(mStream, Configuration.Configuration.BrainStem.CommandReplyTimeout);

                // Example calls to the BrainStem. 
                CommandManager.SoftwareReset(mModuleAddress);
                Thread.Sleep(1000);

                byte servoPosition = CommandManager.GetServoPosition(mModuleAddress, ServoIdType.Servo0);
                Console.WriteLine("Servo Position: {0}", servoPosition);

                int analogValue = CommandManager.ReadA2DInput(mModuleAddress, AnalogToDigitalIoPinType.Pin0, true, false);
                Console.WriteLine("Analog Value: {0} ", analogValue);
            }
            catch (Exception ex)
            {
                Console.WriteLine("Caught Exception: {0}", ex.Message);
            }

            Console.WriteLine("\nPress any key to exit.");
            Console.ReadKey();
        }

        private static byte mModuleAddress;
        private static Stream mStream;
    }
}
